@@ -7,102 +7,169 @@ hformat  <-  function(list.input)
   regressor=list.input$regressor
   varlevel=list.input$varlevel
   group=list.input$group
-  
+  id=list.input$id
+  if (isfield(id,"newId"))
+    id <- id$newId
   nv=c('treatment','parameter','output','regressor','varlevel')
   lv=list(treatment,parameter,output,regressor,varlevel)
   
   #------------------------------------
-  if (!is.null(group)){
-    if (!is.null(names(group))){  
+  if (!is.null(group))
+  {
+    if (!is.null(names(group))) 
       group <- list(group) 
-    }
     N=length(group)
     
-    model.info <- parse.model(list.input$model)
-    for (k in (1:N)){
-      if (!isfield(group[[k]],'size')){
-        group[[k]]$size <- rep(1, length(model.info$level))
-        group[[k]]$level <- model.info$level
-      }else{
-        if (!isfield(group[[k]],'level')){
+    model <- list.input$model
+    
+    if (identical(file_ext(model),"R")) 
+      Rfile <- TRUE 
+    else 
+      Rfile <- FALSE
+    if ( !is.null(model) && exists(model, mode="function") )
+      Rsource <- TRUE
+    else 
+      Rsource <- FALSE
+    
+    if (Rsource || Rfile)
+    {
+      for (k in (1:N))
+      {
+        if (is.null(group[[k]]$size))
+          group[[k]]$size <- 1
+        if (length(group$size)>1)
+          stop("Define group$size as a scalar instead of a vector.")
+        if (!is.null(group[[k]]$level))
+          warning("'level' defined in 'group' is not used with a R model.")
+      }
+    }
+    else
+    {    
+      model.info <- parse.model(model)
+      for (k in (1:N))
+      {
+        if (is.null(group[[k]]$size))
+        {
+          group[[k]]$size <- rep(1, length(model.info$level))
           group[[k]]$level <- model.info$level
-          if (length(group[[k]]$size)!=length(group[[k]]$level))
-            stop("'level' and 'size' defined in 'group' have different lengths.", call.=FALSE)      
-          if (length(model.info$level)>1){
-            msg <- cat("levels of randomization have not been defined in group",k,
-                       '\n levels associated with size = ',group[[k]]$size,' are:\n',
-                       model.info$level,'\n\n')
-            w1 <- warning(msg,call.=FALSE)
+        }
+        else
+        {
+          if (is.null(group[[k]]$level))
+          {
+            if (length(group[[k]]$size)>1)
+              stop("levels of randomization associated to the sizes have not been defined in 'group'", call.=FALSE)      
+            if ("population" %in% model.info$level)
+              group[[k]]$level <- "population"
+            else if ("covariate" %in% model.info$level)
+              group[[k]]$level <- "covariate"
+            else if ("individual" %in% model.info$level)
+              group[[k]]$level <- "individual"
+            else
+              group[[k]]$level <- "longitudinal"
+            
+            if (group[[k]]$level != "longitudinal")
+              warning(paste0("level of randomization has not been defined in group ", k,": '", 
+                             group[[k]]$level, "' is used"), call.=FALSE)      
+            
           }
-        }else{
-          if (isfield(group[[k]],'size') & (length(group[[k]]$size)!=length(group[[k]]$level)))
-            stop("'level' and 'size' defined in 'group' have different lengths.", call.=FALSE)      
-          gk <- group[[k]]$level
-          sk <- rep(1, length(model.info$level))
-          for (jk in (1:length(gk))){
-            rk <- grep(gk[jk],model.info$level)
-            if (length(rk)>0)
-              sk[rk] <- group[[k]]$size[jk]
+          else
+          {
+            if (!is.null(group[[k]]$size) & (length(group[[k]]$size)!=length(group[[k]]$level)))
+              stop("'level' and 'size' defined in 'group' have different lengths.", call.=FALSE)      
+            gk <- group[[k]]$level
+            sk <- rep(1, length(model.info$level))
+            for (jk in (1:length(gk)))
+            {
+              rk <- grep(gk[jk],model.info$level)
+              if (length(rk)>0)
+                sk[rk] <- group[[k]]$size[jk]
+            }
+            group[[k]]$size <- sk
+            group[[k]]$level <- model.info$level
           }
-          group[[k]]$size <- sk
-          group[[k]]$level <- model.info$level
         }
       }
     }
-  }else{
-    N=NULL
   }
+  else
+    N <- nrow(list.input$id)
+  # N=NULL
   
-  for (k in seq(1,length(lv))) {
+  Nid<-NULL
+  for (k in seq(1,length(lv))) 
+  {
     lvk <- lv[[k]]
-    if (!is.null(names(lvk))){  
+    if (!is.null(names(lvk)))
+    {  
       lvk <- list(lvk) 
       lv[[k]] <- lvk
     }
-    for (j in seq(1,length(lvk))) {
-      lvkj <- lvk[[j]]
-      dkj <- dim(lvkj)
-      #     if (!is.null(dkj)){
-      #   N=c(N,dkj[1])
-      if (isfield(lvkj,"id")){
-        N <- c(N,length(unique(lvkj$id)))
-      }else if(isfield(lvkj,"design")){
-        N <- c(N,length(unique(lvkj$design$id)))
-      }else if(isfield(lvkj,"time")){
-        if (isfield(lvkj$time,"id")){
-          N <- c(N,length(unique(lvkj$time$id)))
-          names(lvkj)[names(lvkj)=="time"]<-"design"
+    if (length(lvk)>0)
+    {
+      for (j in seq(1,length(lvk))) 
+      {
+        lvkj <- lvk[[j]]
+        # dkj <- dim(lvkj)
+        #     if (!is.null(dkj)){
+        #   N=c(N,dkj[1])
+        if (isfield(lvkj,"id"))
+        {
+          # N <- c(N,length(unique(lvkj$id)))
+          # lu <- unique(lvkj$id)
+          lvkj$id <- match(lvkj$id, id)
+          Nid <- c(Nid,lvkj$id)
+        }
+        else if(isfield(lvkj,"design"))
+        {
+          # lu <- unique(lvkj$design$id)
+          lvkj$design$id <- match(lvkj$design$id, id)
+          Nid <- c(Nid,lvkj$design$id)
+          #             names(lvkj)[names(lvkj)=="design"]<-"time"
           lv[[k]][[j]]=lvkj
         }
-      }else if(isfield(lvkj,'header')){
-        warning("deprecated syntax:  use 'colNames' instead of 'header'",immediate.=TRUE)
-        names(lvkj)[names(lvkj)=="header"]<-"colNames"
+        else if(isfield(lvkj,"time"))
+        {
+          if (isfield(lvkj$time,"id"))
+          {
+            # lu <- unique(lvkj$time$id)
+            lvkj$time$id <- match(lvkj$time$id, id)
+            Nid <- c(Nid,lvkj$time$id)
+          }
+        }
+        else if(isfield(lvkj,'header'))
+        {
+          warning("deprecated syntax:  use 'colNames' instead of 'header'",immediate.=TRUE)
+          names(lvkj)[names(lvkj)=="header"]<-"colNames"
+        }
+        
+        if (isfield(lvkj,'colNames'))
+        {
+          #N=c(N,length(unique(lvkj$value[,1]))) #assuming that first column = id
+          Nid=c(Nid,lvkj$value[,1]) #assuming that first column = id
+        }
         lv[[k]][[j]]=lvkj
       }
-      
-      if (isfield(lvkj,'colNames')){
-        N=c(N,length(unique(lvkj$value[,1]))) #assuming that first column = id
-      }
     }
   }    
+  Nid = length(unique(Nid))
   
-  if (is.null(N)){
+  if (is.null(N))
+    N <- Nid 
+  
+  if (is.null(N)||N==0)
     N=1
-  }else{
-    N=unique(N)
-    if (length(N)>1){
-      stop("different numbers of individuals and/or groups are defined \n",call.="FALSE")
-    }
-  }
+  else
+    N = unique(N)
   
-  if (is.null(group)){
+  if (is.null(group))
+  {
     group <- vector("list",N)
     for (i in seq(1,N))
       group[[i]] <- list(size=1)
   }
   
   id.ori=list()
-  
   #---  parameters
   iv <- which(nv=="parameter")
   parameter <- vector("list", N)
@@ -111,9 +178,11 @@ hformat  <-  function(list.input)
     if (length(r$id)>0) {id.ori=c(id.ori,r$id)}
     parameter <- r$parameter
   }
-  for (i in seq(1,N)){
+  for (i in seq(1,N))
+  {
     gi <- group[[i]]
-    if (isfield(gi,'parameter')) {
+    if (isfield(gi,'parameter')) 
+    {
       r <- format.parameter(parameter,gi$parameter,i)
       parameter <- r$parameter
       group[[i]]$parameter <- NULL
@@ -123,8 +192,9 @@ hformat  <-  function(list.input)
   list.output=list(parameter=parameter)  
   
   #---  outputs
-  output <- list(individual=vector("list", N),group=vector("list", N))
-  for (i in seq(1,N)){
+  output <- list(individual=vector("list", N),group=vector("list", N), id=id)
+  for (i in seq(1,N))
+  {
     output$individual[[i]]=list(name=NULL,time=NULL)
     output$group[[i]]=list(name=NULL)
   }
@@ -132,9 +202,11 @@ hformat  <-  function(list.input)
   r <- format.output(output,lv[[iv]],seq(1,N))
   if (length(r$id)>0) {id.ori=c(id.ori,r$id)}
   output <- r$output
-  for (i in seq(1,N)){
+  for (i in seq(1,N))
+  {
     gi <- group[[i]]
-    if (isfield(gi,'output')) {
+    if (isfield(gi,'output')) 
+    {
       r <- format.output(output,gi$output,i)
       output <- r$output
     }
@@ -145,35 +217,44 @@ hformat  <-  function(list.input)
   
   #---  treatments
   iv <- which(nv=="treatment")
-  if (!is.null(lv[[iv]])){
+  if (!is.null(lv[[iv]]))
+  {
     r <- format.treatment(lv[[iv]],seq(1,N))
-    if (length(r$id)>0) {id.ori=c(id.ori,r$id)}
+    if (length(r$id)>0) 
+      id.ori=c(id.ori,r$id)
     treatment <- r$treatment
   }
-  for (i in seq(1,N)){
+  for (i in seq(1,N))
+  {
     gi <- group[[i]]
-    if (isfield(gi,'treatment')) {
+    if (isfield(gi,'treatment')) 
+    {
       r <- format.treatment(gi$treatment,i)
       pgi <- r$treatment
       treatment <- c(treatment, pgi)
       group[[i]]$treatment <- NULL
     }
   }
-  if (!is.null(treatment)){
+  if (!is.null(treatment))
+  {
     r <- merge.treatment(treatment,N)
     list.output <- c(list.output, r)
   }
   
   #---  regressor
   iv <- which(nv=="regressor")
-  if (!is.null(lv[[iv]])){
+  if (!is.null(lv[[iv]]))
+  {
     r <- format.regressor(lv[[iv]],seq(1,N))
-    if (length(r$id)>0) {id.ori=c(id.ori,r$id)}
+    if (length(r$id)>0) 
+      id.ori=c(id.ori,r$id)
     regressor <- r$regressor
   }
-  for (i in seq(1,N)){
+  for (i in seq(1,N))
+  {
     gi <- group[[i]]
-    if (isfield(gi,'regressor')) {
+    if (isfield(gi,'regressor')) 
+    {
       r <- format.regressor(gi$regressor,i)
       pgi <- r$regressor
       regressor <- c(regressor, pgi)
@@ -185,14 +266,17 @@ hformat  <-  function(list.input)
   
   #---  varlevel
   iv <- which(nv=="varlevel")
-  if (!is.null(lv[[iv]])){
+  if (!is.null(lv[[iv]]))
+  {
     r <- format.varlevel(lv[[iv]],seq(1,N))
-    if (length(r$id)>0) {id.ori=c(id.ori,r$id)}
+    if (length(r$id)>0) 
+      id.ori=c(id.ori,r$id)
     varlevel <- r$varlevel
   }
   for (i in seq(1,N)){
     gi <- group[[i]]
-    if (isfield(gi,'varlevel')) {
+    if (isfield(gi,'varlevel')) 
+    {
       r <- format.varlevel(gi$varlevel,i)
       pgi <- r$varlevel
       varlevel <- c(varlevel, pgi)
@@ -203,15 +287,17 @@ hformat  <-  function(list.input)
     list.output$varlevel <- merge.varlevel(varlevel,N)
   
   #--------------------
-  if (length(unique(id.ori))>0){
-    if (length(unique(id.ori))>1){
-      stop("\n\nid's should be the same in the different input arguments (parameters, treatment, output,...)\n")
-    }else{
-      list.output$id.ori <- unique(id.ori)[[1]]
+  if (length(unique(id.ori))>0)
+  {
+    if (length(unique(id.ori))>1)
+    {
+      #stop("\n\nid's should be the same in the different input arguments (parameters, treatment, output,...)\n")
     }
-  }else{
-    list.output$id.ori <- NULL
+    else
+      list.output$id.ori <- unique(id.ori)#[[1]]
   }
+  else
+    list.output$id.ori <- NULL
   
   #--------------------
   list.output$group=group
@@ -222,40 +308,58 @@ hformat  <-  function(list.input)
 format.treatment <- function(treatment,uN) 
 {
   N <- length(uN)
-  if (!is.null(names(treatment))){  
+  if (!is.null(names(treatment))) 
     treatment <- list(treatment) 
-  }
-  id.ori <- list()
-  for (k in seq(1,length(treatment))){
+  id.ori <- NULL
+  for (k in seq(1,length(treatment)))
+  {
     trtk <- treatment[[k]]
-    if (isfield(trtk,'colNames')){
+    if (isfield(trtk,'colNames'))
+    {
       pp <- as.data.frame(trtk$value)
       names(pp) <- trtk$colNames
       trtk=pp
-    }else if (!is.data.frame(trtk)){
-      trtk <- as.data.frame(trtk)
+    }
+    else if (!is.data.frame(trtk))
+    {
+      trtk <- as.data.frame(trtk,stringsAsFactors =FALSE)
       n <- nrow(trtk)
       trtk <- trtk[rep(1:n,each=N),] 
       trtk$id <- rep(uN,n)
-    }else{
+    }
+    else
+    {
       idk <- sort(unique(trtk$id))
-      if (any(idk != uN)) {
-        id.ori[[length(id.ori)+1]]<-idk 
-        trtk$id <- match(trtk$id, idk)
-      }
+      # if (any(idk != uN)) 
+      #id.ori[[length(id.ori)+1]]<-idk
+      # id.ori<-c(id.ori,idk)
+      # trtk$id <- match(trtk$id, idk)
       
     }
     names(trtk)[names(trtk)=="amt"] <- "amount"
     names(trtk)[names(trtk)=="adm"] <- "type"
-    if (isfield(trtk,"tinf")){
-      names(trtk)[names(trtk)=="tinf"] <- "rate"
-      trtk$rate <- trtk$amount/trtk$rate
+    
+    if (!is.null(trtk$rate))
+    {
+      trtkrate=rep(Inf,length(trtk$rate))
+      irate <- (trtk$rate!=".")
+      trtkrate[irate]=as.numeric(as.character(trtk$rate[irate]))
+      trtk$rate <- trtkrate
     }
-    if (!isfield(trtk,"rate"))
+    if (!is.null(trtk$tinf))
+    {
+      names(trtk)[names(trtk)=="tinf"] <- "rate"
+      trtkrate=rep(Inf,length(trtk$rate))
+      irate <- trtk$rate!="."
+      trtkrate[irate]=trtk$amount[irate]/as.numeric(as.character(trtk$rate[irate]))
+      trtk$rate <- trtkrate
+    }
+    if (is.null(trtk$rate))
       trtk$rate <- Inf
     treatment[[k]] <- trtk   
   }
-  r <- list(treatment=treatment, id=id.ori)
+  #r <- list(treatment=treatment, id=id.ori)
+  r <- list(treatment=treatment, id=unique(id.ori))
   return(r)
 }
 #--------------------------
@@ -264,37 +368,41 @@ merge.treatment <- function(treatment,N)
   p <- vector("list",N)
   ptr <- treatment[[1]]
   np <- length(treatment)
-  if (np>1){
-    for (j in seq(2,np)){
+  if (np>1)
+  {
+    for (j in seq(2,np))
       ptr=rbind(ptr,treatment[[j]])
-    }
   }
   
-  if (isfield(ptr,"target")){
+  if (isfield(ptr,"target"))
+  {
+    ptr$target = as.factor(ptr$target)
     list.target <- levels(ptr$target)
     nt <- length(list.target)
     depot <- vector("list", nt)
-    for (k in seq(1,nt)){
+    for (k in seq(1,nt))
       depot[[k]] <- list(type=k, target=list.target[k])
-    }
     ptr$type    <- as.numeric(ptr$target)
     ptr$target  <- NULL
-  }else{
+  }
+  else
+  {
     depot<-NULL
   }
   
-  for (i in seq(1,N)){
+  for (i in seq(1,N))
+  {
     pi <- NULL
     ij <- which(ptr$id==i)
-    if (length(ij)>0) {
+    if (length(ij)>0) 
       pi <- ptr[ij,]
-    }
     pi$id=NULL
     
     if (!isfield(pi,"type"))
       pi$type <- 1
     
-    if (!is.null(pi$time)){
+    if (!is.null(pi$time))
+    {
       pi <- pi[with(pi,order(time)),]
       p[[i]] <- as.list(pi)
     }
@@ -307,64 +415,75 @@ merge.treatment <- function(treatment,N)
 format.parameter <- function(parameter,param,uN) 
 {
   N <- length(uN)
-  if (!is.null(names(param))){  
+  if (!is.null(names(param)))
     param=list(param) 
-  }
-  id.ori <- list()
-  
-  for (k in seq(1,length(param))){
+  param <- param[!unlist(lapply(param, is.null))]
+  #id.ori <- list()
+  id.ori <- NULL
+  for (k in seq(1,length(param)))
+  {
     paramk <- param[[k]]
-    if (isfield(paramk,'colNames')){
+    if (isfield(paramk,'colNames'))
+    {
       pp=as.data.frame(paramk$value)
       names(pp)=paramk$colNames
       paramk=pp
     }
     id <- uN
-    if (!is.data.frame(paramk)){
-      if (!is.list(paramk)){
+    if (!is.data.frame(paramk))
+    {
+      if (!is.list(paramk))
         paramk <- list(name=names(paramk),value=as.vector(paramk))
-      }
       pk <- rep(paramk$value,N)
       pk <- t(matrix(pk,ncol=N))
       pk <- cbind(uN,pk)
       pk <- data.frame(pk)
       names(pk) <- c('id',paramk$name)
       paramk <- pk
-    }else{
-      if (!isfield(paramk,"id")){
+    }
+    else
+    {
+      if (!isfield(paramk,"id"))
+      {
         paramk <- paramk[rep(1,each=N),] 
         paramk$id <- uN
-      }else{
+      }
+      else
+      {
         idk <- sort(unique(paramk$id))
-        if (any(idk != uN)){
-          id.ori[[length(id.ori)+1]]<-idk    
+        if (any(idk != uN))
+        {
+          #id.ori[[length(id.ori)+1]]<-idk    
+          id.ori<-c(id.ori,idk) 
           paramk$id <- match(paramk$id, idk)
         }
       }
     }
-    for (i in uN){
+    for (i in uN)
+    {
       pki <- paramk[paramk$id==i,]
       pki$id <- i
-      if (is.null(parameter[[i]])){
+      if (is.null(parameter[[i]]))
         parameter[[i]] <- pki
-      }else{
+      else
         parameter[[i]] <- merge(parameter[[i]],pki)
-      }
     }
   }
-  r <- list(parameter=parameter, id=id.ori)
+  # r <- list(parameter=parameter, id=id.ori)
+  r <- list(parameter=parameter, id=unique(id.ori))
   return(r)
 }
 #-----------------------------
 merge.parameter <- function(parameter) 
 {
-  if (!is.null(parameter[[1]])){
+  if (!is.null(parameter[[1]]))
+  {
     N <- length(parameter)
     pp <- parameter[[1]]
-    if (N>1){
-      for (i in seq(2,N)){
+    if (N>1)
+    {
+      for (i in seq(2,N))
         pp <- rbind(pp, parameter[[i]])
-      }
     }
     parameter <- pp[with(pp,order(id)),]
     #    parameter$id <- NULL  
@@ -378,47 +497,63 @@ merge.parameter <- function(parameter)
 format.output <- function(output, out,uN) 
 {
   N <- length(uN)
-  if (!is.null(names(out))){  
+  if (!is.null(names(out)))
     out=list(out) 
-  }
   
   ioutput <- output$individual
   goutput <- output$group
-  id.ori <- list()
-  
-  for (k in seq(1,length(out))){
+  id.ori <- NULL
+  for (k in seq(1,length(out)))
+  {
     outk <- out[[k]]
+    if (is.data.frame(outk))
+    {
+      n.outk <- names(outk)
+      outk <- list(name=n.outk[!(n.outk %in% c("id","time"))], 
+                   time = outk[,c("id","time")])
+    }
     if (!isfield(outk,"name"))
       outk <- list(name=outk)
     okname <- unlist(outk$name)
     nok <- length(okname)
-    if (isfield(outk,'colNames')){
+    if (isfield(outk,'colNames'))
+    {
       pp=as.data.frame(outk$value)
       names(pp)=outk$colNames
       outk$design=pp
     }
-    if (isfield(outk,'design')){      
-      id <- sort(unique(outk$design$id))
-      if (any(id != uN)) id.ori[[length(id.ori)+1]]<-id        
-      for (i in uN){
-        ji <- which(outk$design$id == id[i])
-        ti <- sort(outk$design$time[ji])
+    if (!is.null(outk$time) && ("id" %in% names(outk$time)))
+    {      
+      # id <- sort(unique(as.factor(outk$time$id)))
+      # id <- sort(unique(outk$time$id))
+      # id <- output$id
+      # if (length(id) !=length(uN) ||  any(id != uN))
+      #   id.ori<-sort(unique(c(id.ori,id)))
+      # idk <- match(id,levels(id))
+      # idk <- match(sort(unique(outk$time$id)),id)
+      idk <- match(sort(unique(outk$time$id)),uN)
+      for (i in seq(1:length(idk)))
+      {
+        ji <- which(outk$time$id == idk[i])
+        ti <- sort(outk$time$time[ji])
         oitime <- vector("list" , nok)
-        for (j in seq(1,nok)){
+        for (j in seq(1,nok))
           oitime[[j]] <- ti
-        }
-        goutput[[i]]$name=c(goutput[[i]]$name,okname)
-        ioutput[[i]]$name=c(ioutput[[i]]$name,okname)
-        ioutput[[i]]$time=c(ioutput[[i]]$time,oitime)
+        ik <- idk[i]
+        goutput[[ik]]$name=c(goutput[[ik]]$name,okname)
+        ioutput[[ik]]$name=c(ioutput[[ik]]$name,okname)
+        ioutput[[ik]]$time=c(ioutput[[ik]]$time,oitime)
       }
-      
-    }else{ 
-      for (i in uN){
-        if (isfield(outk,"time")){
+    }
+    else
+    { 
+      for (i in uN)
+      {
+        if (isfield(outk,"time"))
+        {
           oitime <- vector("list" , nok)
-          for (j in seq(1,nok)){
+          for (j in seq(1,nok))
             oitime[[j]] <- sort(outk$time)
-          }
           ioutput[[i]]$time=c(ioutput[[i]]$time,oitime)
           ioutput[[i]]$name=c(ioutput[[i]]$name,okname)
         }
@@ -428,7 +563,8 @@ format.output <- function(output, out,uN)
   }
   output$individual <- ioutput 
   output$group <- goutput
-  r <- list(output=output, id=id.ori)
+  #r <- list(output=output, id=id.ori)
+  r <- list(output=output, id=unique(id.ori))
   return(r)
 }
 
@@ -440,14 +576,20 @@ format.regressor <- function(reg, uN)
     reg=list(reg) 
   
   regressor <- vector("list",length(reg))
-  id.ori <- list()
-  for (k in seq(1,length(reg))){
+  #id.ori <- list()
+  id.ori <- NULL
+  for (k in seq(1,length(reg)))
+  {
     regk <- reg[[k]]
-    if (!is.data.frame(regk)){
-      if (isfield(regk,'colNames')){
+    if (!is.data.frame(regk))
+    {
+      if (isfield(regk,'colNames'))
+      {
         mk <- regk$value
         colNames=regk$colNames
-      }else{
+      }
+      else
+      {
         nk <- length(regk$time)
         idk <- rep(uN,each=nk)
         mk <- cbind(regk$time,regk$value)
@@ -457,37 +599,53 @@ format.regressor <- function(reg, uN)
       }
       regk <- data.frame(mk)
       names(regk) <- colNames
-    }else{
-      colNames <- names(regk)
-      #      mk <- data.matrix(regk)
-      mk <- matrix(as.numeric(unlist(regk)),nrow=nrow(regk))
-      mk <- as.data.frame(mk)
-      names(mk) <- colNames
-      idk <- sort(unique(mk$id))
-      if (any(idk != uN)) {
-        id.ori[[length(id.ori)+1]]<-idk 
-        mk$id <- match(mk$id, idk)
-      }
     }
+    else
+    {
+      #       colNames <- names(regk)
+      #       #      mk <- data.matrix(regk)
+      #       mk <- matrix(as.numeric(unlist(regk)),nrow=nrow(regk))
+      #       mk <- as.data.frame(mk)
+      #       names(mk) <- colNames
+      mk <- regk
+      if (is.null(mk$id))
+        mk <- cbind(list(id=1),mk)
+      # idk <- sort(unique(mk$id))
+      # if (any(idk != uN)) 
+      # {
+      #id.ori[[length(id.ori)+1]]<-idk 
+      # id.ori<-c(id.ori,idk) 
+      # mk$id <- match(mk$id, idk)
+      # }
+      regk <- mk
+    }
+    # idu <- unique(regk$id)
+    # regk$id <- match( regk$id , idu )
+    regk <- regk[order(regk$id,regk$time),]
     regressor[[k]] <- regk
   }
-  r <- list(regressor=regressor, id=id.ori)
+  # r <- list(regressor=regressor, id=id.ori)
+  r <- list(regressor=regressor, id=unique(id.ori))
   return(r)
 }
 
 #-----------------------------
 merge.regressor <- function(reg, N) 
 {
-  if (length(reg)>1){
+  if (length(reg)>1)
+  {
     r <- NULL
-    for (i in seq(1,N)){
+    for (i in seq(1,N))
+    {
       ri <- NULL
-      for (k in seq(1,length(reg))){
+      for (k in seq(1,length(reg)))
+      {
         rik <- reg[[k]][reg[[k]]$id==i,]
         if (nrow(rik)>0)
-          if (is.null(ri)){
+          if (is.null(ri))
             ri <- rik
-          }else{
+          else
+          {
             ri <- merge(ri,rik,
                         by.x=c("id","time"), by.y=c("id","time"),
                         all.x=TRUE,all.y=TRUE) 
@@ -496,11 +654,9 @@ merge.regressor <- function(reg, N)
       }
       r <- rbind(r,ri)
     }
-  }else{
-    r <- reg[[1]]
   }
-  
-  #  m <- data.matrix(r)
+  else
+    r <- reg[[1]]
   m <- matrix(as.numeric(unlist(r)),nrow=nrow(r))
   colNames <- names(r)
   colTypes <- rep("x",length(colNames))
@@ -535,13 +691,15 @@ format.varlevel <- function(var, uN)
       }
       vark <- data.frame(mk)
       names(vark) <- colNames
-    }else{
-      idk <- sort(unique(vark$id))
-      if (any(idk != uN)) {
-        id.ori[[length(id.ori)+1]]<-idk 
-        vark$id <- match(vark$id, idk)
-      }
     }
+    # else
+    #   {
+    #   idk <- sort(unique(vark$id))
+    #   if (any(idk != uN)) {
+    #     id.ori[[length(id.ori)+1]]<-idk 
+    #     vark$id <- match(vark$id, idk)
+    #   }
+    # }
     varlevel[[k]] <- vark
   }
   r <- list(varlevel=varlevel, id=id.ori)
@@ -551,29 +709,37 @@ format.varlevel <- function(var, uN)
 #-----------------------------
 merge.varlevel <- function(var, N) 
 {
-  if (length(var)>1){
+  if (length(var)>1)
+  {
     r <- NULL
-    for (i in seq(1,N)){
+    for (i in seq(1,N))
+    {
       ri <- NULL
-      for (k in seq(1,length(var))){
+      for (k in seq(1,length(var)))
+      {
         rik <- var[[k]][var[[k]]$id==i,]
         if (nrow(rik)>0)
-          if (is.null(ri)){
+          if (is.null(ri))
+          {
             ri <- rik
-          }else{
-            ri <- merge(ri,rik,
-                        by.x=c("id","time"), by.y=c("id","time"),
-                        all.x=TRUE,all.y=TRUE) 
-            for (j in seq(1,ncol(ri))){
-              ij <- c(which(!is.na(ri[,j])),nrow(ri)+1)
-              
-              for (l in seq(1,length(ij)-1)){
-                ijl <- ij[l]
-                if (ij[l+1]-ijl>1)
-                  ri[seq(ijl+1,ij[l+1]-1),j] <- ri[ijl,j]
-              }
+          }
+        else
+        {
+          ri <- merge(ri,rik,
+                      by.x=c("id","time"), by.y=c("id","time"),
+                      all.x=TRUE,all.y=TRUE) 
+          for (j in seq(1,ncol(ri)))
+          {
+            ij <- c(which(!is.na(ri[,j])),nrow(ri)+1)
+            
+            for (l in seq(1,length(ij)-1))
+            {
+              ijl <- ij[l]
+              if (ij[l+1]-ijl>1)
+                ri[seq(ijl+1,ij[l+1]-1),j] <- ri[ijl,j]
             }
           }
+        }
       }
       r <- rbind(r,ri)
     }
@@ -598,16 +764,17 @@ parse.model <- function(model)
   lines      = readLines(con, warn=FALSE)
   close(con)
   ip <- grep(";",lines, fixed=TRUE)
-  if (length(ip)>0){
-    for (k in (1:length(ip))){
+  if (length(ip)>0)
+  {
+    for (k in (1:length(ip)))
+    {
       ll <- lines[ip[k]]
       il <- regexpr(";",ll)
       il <- il[1]
-      if (il>1){
+      if (il>1)
         lines[ip[k]] <- substr(ll,start=1,stop=(il-1))
-      }else{
+      else
         lines[ip[k]] <- ''
-      }
     }
   }
   
